from __future__ import absolute_import # Python 2.5+
import unittest2
import webtest
import urllib
from tarn.case.base import BaseTestCase


class WebTestCase(BaseTestCase):
    def build_wsgi_application(self):
        # You can override this to provide an application, if you like.
        return None
        
    def setUp(self):
        super(WebTestCase, self).setUp()
        self._app = None
        self._last_response = None
        
    def tearDown(self):
        self._app = None
        self._last_response = None
        super(WebTestCase, self).tearDown()
        
    @property
    def last_response(self):
        return self._last_response
        
    @property
    def app(self):
        """Returns a webapp TestApp based either on the app supplied by 
           build_wsgi_application, or by calling tarn_hooks.build_wsgi_application() if present."""
        if (not hasattr(self, '_app')) or (self._app is None):
            wsgi_application = self.build_wsgi_application()
            if wsgi_application is None:
                try:
                    from tarn_hooks import build_wsgi_application as hook_build_wsgi_application
                    wsgi_application = hook_build_wsgi_application()
                except:
                    pass
            if wsgi_application is None:
                self.fail("To create a test application, you must override build_wsgi_application() in your test case, or provide a build_wsgi_application() method in your tarn_hooks.py.")
            self._app = webtest.TestApp(wsgi_application)
        return self._app
    
    def assertOK(self, response = None):
        response = response or self._last_response
        self.assertEqual(response.status_int, 200, "Expected status code 200, got %r" % response.status_int)
    
    def assertOk(self, response = None): # I hate typing.
        self.assertOK(response = response) 
        
    def assertNotOK(self, response = None):
        response = response or self._last_response
        self.assertNotEqual(response.status_int, 200, "Expected status code was anything other than 200 but we got 200.")
        
    def assertNotOk(self, response = None): # I hate typing.
        self.assertNotOK(response = response)

    def assertNotFound(self, response = None):
        response = response or self._last_response
        return self.assertEqual(response.status_int, 404, "Expected status code 404, got %r" % response.status_int)
    
    def assertForbidden(self, response = None):
        response = response or self._last_response        
        return self.assertEqual(response.status_int, 403, "Expected status code 403, got %r" % response.status_int)
        
    def assertRedirect(self, response = None, to = None):
        response = response or self._last_response        
        self.assertTrue(response.status_int in [301, 302], "Expected a redirect status code, got %r" % response.status_int)
        if to is not None:
            self.assertEqual(response.location, "http://localhost:80%s" % to, "Expected a redirect to %s, but got %s instead" % (to, response.location))
    
    def get(self, url, *args, **kwargs):
        kwargs.setdefault('status', '*')
        self._last_response = self.app.get(url, *args, **kwargs)
        return self._last_response
    
    def post(self, url, data, *args, **kwargs):
        kwargs.setdefault('status', '*')
        encoded_data = urllib.urlencode(data)
        self._last_response = self.app.post(url, encoded_data, *args, **kwargs)
        return self._last_response
        
    def delete(self, url, *args, **kwargs):
        kwargs.setdefault('status', '*')
        self._last_response = self.app.delete(url, *args, **kwargs)
        return self._last_response
        
    def put(self, url, *args, **kwargs):
        kwargs.setdefault('status', '*')
        self._last_response = self.app.put(url, *args, **kwargs)
        return self._last_response
        
    def get_cookie(self, key, default = None):
        return self.app.cookies.get(key, default)
        
    def set_cookie(self, key, value):
        self.app.cookies[key] = value
        
    def find_in_html(self, response = None, name = None, attrs = {}, recursive = True, substring = None, prefix = None, text = None, limit = None, **kwargs):
        response = response or self._last_response
        
        found_items = response.html.findAll(name = name, attrs = attrs, recursive = recursive, text = text, limit = limit, **kwargs)
        
        if substring is not None:
            found_items = [found_item for found_item in found_items if substring in found_item.text]
        
        if prefix is not None:
            found_items = [found_item for found_item in found_items if found_item.text.startswith(prefix)]
            
        return found_items
        
    def assertInHtml(self, response = None, name = None, attrs = {}, recursive = True, substring = None, prefix = None, text = None, limit = None, **kwargs):
        response = response or self._last_response
        found_items = self.find_in_html(response, name = name, attrs = attrs, recursive = recursive, substring = substring, prefix = prefix, text = text, limit = limit, **kwargs)
        self.assertTrue(self.get_length(found_items) > 0, "Could not find an HTML item mathing the constraints.")
    
    def assertNotInHtml(self, response = None, name = None, attrs = {}, recursive = True, substring = None, prefix = None, text = None, limit = None, **kwargs):
        response = response or self._last_response
        found_items = self.find_in_html(response, name = name, attrs = attrs, recursive = recursive, substring = substring, prefix = prefix, text = text, limit = limit, **kwargs)
        self.assertTrue(self.get_length(found_items) == 0, "Found an HTML item matching the constrations, but didn't want to find it.")
        
    def assertHtmlItemCount(self, response = None, count = 0, name = None, attrs = {}, recursive = True, substring = None, prefix = None, text = None, limit = None, **kwargs):
        response = response or self._last_response
        found_items = self.find_in_html(response, name = name, attrs = attrs, recursive = recursive, substring = substring, prefix = prefix, text = text, limit = limit, **kwargs)
        found_count = self.get_length(found_items)
        self.assertTrue(found_count == conut, "Found %r matching HTML items, but expected %r" % (found_count, count))
        
    def assertLink(self, response = None, attrs = {}, recursive = True, substring = None, prefix = None, text = None, limit = None, **kwargs):
        self.assertInHtml(response = response, name = "a", attrs = attrs, recursive = recursive, substring = substring, prefix = prefix, text = text, limit = limit, **kwargs)

    def follow_redirect(self, response = None):
        response = response or self._last_response
        self.assertRedirect(response)
        self._last_response = response.follow(status = '*')
        return self._last_response
        
    def show_in_browser(self, response = None):
        response = response or self._last_response
        if response is not None:
            response.showbrowser()
    
    @property
    def forms(self):
        return self._last_response.forms if (self._last_response is not None) else None
        
    @property
    def form(self):
        return self._last_response.form if (self._last_response is not None) else None
        
    def submit_form(self, *args, **kwargs):
        self._last_response = self.form.submit(*args, **kwargs)
        return self._last_response
        
    @property
    def title(self):
        return self._last_response.html.html.head.title
        
    def click(self, response = None, description = None, linkid = None, href = None, anchor = None, index = None, verbose = False):
        response = response or self._last_response
        self._last_response = response.click(description = description, linkid = linkid, href = href, anchor = anchor, index = index, verbose = verbose)
        return self._last_response

    def assertTitle(self, title = None, substring = None):
        if title is not None:
            self.assertEqual(self.title.text, title, "Titles do not match: expected '%s' but got '%s'" % (title, self.title))
        if substring is not None:
            self.assertTrue(substring in self.title.text, "Titles do not match: expected substring '%s' but got title '%s'" % (substring, self.title))
